/**
 * @file
 * JavaScript for the sentiment analysis dashboard.
 */
(function ($, Drupal, drupalSettings) {
  'use strict';

  Drupal.behaviors.aiWebformSentimentDashboard = {
    attach: function (context, settings) {
      const trendCanvas = document.getElementById('sentiment-trend-chart-' + drupalSettings.ai_webform_sentiment.webform_id);
      const barCanvas = document.getElementById('sentiment-bar-chart-' + drupalSettings.ai_webform_sentiment.webform_id);
      const totalsCanvas = document.getElementById('sentiment-totals-chart-' + drupalSettings.ai_webform_sentiment.webform_id);
      const tagsCanvas = document.getElementById('tags-polar-chart-' + drupalSettings.ai_webform_sentiment.webform_id);

      if (drupalSettings.ai_webform_sentiment) {
        // Create trend chart (line chart with averages only)
        if (trendCanvas && trendCanvas.getContext) {
          trendCanvas.once = trendCanvas.once || {};

          if (!trendCanvas.once.sentimentTrendChart) {
            trendCanvas.once.sentimentTrendChart = true;
            const labels = drupalSettings.ai_webform_sentiment.days;
            const trendDatasets = [];

            // Add daily averages as a line chart.
            if (drupalSettings.ai_webform_sentiment.daily_averages) {
              trendDatasets.push({
                label: Drupal.t('Daily Average'),
                data: drupalSettings.ai_webform_sentiment.daily_averages,
                fill: true,
                borderColor: 'rgb(34, 139, 34)',
                backgroundColor: 'rgba(34, 139, 34, 0.2)',
                borderWidth: 3,
                borderDash: [5, 5],
                tension: 0.1,
                pointRadius: 4,
                pointHoverRadius: 6
              });
            }

            const trendConfig = {
              type: 'line',
              data: {
                labels: labels,
                datasets: trendDatasets
              },
              options: {
                responsive: true,
                scales: {
                  y: {
                    min: 0,
                    max: 5,
                    beginAtZero: true
                  }
                },
                plugins: {
                  legend: {
                    position: 'top',
                  },
                  title: {
                    display: true,
                    text: Drupal.t('Daily Average Sentiment Trend')
                  }
                }
              }
            };

            new Chart(trendCanvas.getContext('2d'), trendConfig);
          }
        }

        // Create bar chart (sentiment counts)
        if (barCanvas && barCanvas.getContext) {
          barCanvas.once = barCanvas.once || {};

          if (!barCanvas.once.sentimentBarChart) {
            barCanvas.once.sentimentBarChart = true;

            const labels = drupalSettings.ai_webform_sentiment.days;
            const sentimentCount = drupalSettings.ai_webform_sentiment.sentiment_count;
            const barDatasets = [];

            // Define colors for different sentiments.
            const colors = [
              'rgb(75, 192, 192)',   // teal
              'rgb(255, 99, 132)',   // red
              'rgb(255, 205, 86)',   // yellow
              'rgb(54, 162, 235)',   // blue
              'rgb(153, 102, 255)',  // purple
              'rgb(255, 159, 64)',   // orange
              'rgb(201, 203, 207)'   // gray
            ];

            // Create array of sentiment data with labels for sorting.
            const sentimentData = [];
            for (let i = 0; i < sentimentCount; i++) {
              sentimentData.push({
                label: drupalSettings.ai_webform_sentiment['label_' + i],
                data: drupalSettings.ai_webform_sentiment['data_' + i],
                index: i
              });
            }

            // Sort numerically by label value.
            sentimentData.sort((a, b) => parseFloat(a.label) - parseFloat(b.label));

            // Create datasets in sorted order.
            sentimentData.forEach((sentiment, index) => {
              barDatasets.push({
                label: sentiment.label,
                data: sentiment.data,
                fill: false,
                borderColor: colors[sentiment.index % colors.length],
                backgroundColor: colors[sentiment.index % colors.length] + '40', // Light transparency for area under line
                tension: 0.1,
                borderWidth: 2,
                pointRadius: 3,
                pointHoverRadius: 5
              });
            });

            const barConfig = {
              type: 'line',
              data: {
                labels: labels,
                datasets: barDatasets
              },
              options: {
                responsive: true,
                scales: {
                  y: {
                    beginAtZero: true
                    // No max limit - let it scale automatically
                  }
                },
                plugins: {
                  legend: {
                    position: 'top',
                  },
                  title: {
                    display: true,
                    text: Drupal.t('Sentiment Distribution by Day')
                  }
                }
              }
            };

            new Chart(barCanvas.getContext('2d'), barConfig);
          }
        }

        // Create totals chart (bar chart with sentiment totals)
        if (totalsCanvas && totalsCanvas.getContext && drupalSettings.ai_webform_sentiment.sentiment_totals) {
          totalsCanvas.once = totalsCanvas.once || {};

          if (!totalsCanvas.once.sentimentTotalsChart) {
            totalsCanvas.once.sentimentTotalsChart = true;

            const sentimentTotals = drupalSettings.ai_webform_sentiment.sentiment_totals;
            const totalsLabels = [];
            const totalsData = [];
            const totalsColors = [];

            // Define colors for sentiment values.
            const colorMap = {
              '0': 'rgb(220, 53, 69)',    // red for very negative
              '1': 'rgb(255, 159, 64)',   // orange for negative
              '2': 'rgb(255, 205, 86)',   // yellow for neutral
              '3': 'rgb(75, 192, 192)',   // teal for positive
              '4': 'rgb(54, 162, 235)',   // blue for very positive
              '5': 'rgb(40, 167, 69)'     // green for extremely positive
            };

            // Sort sentiment values numerically and prepare data.
            const sortedSentiments = Object.keys(sentimentTotals).sort((a, b) => parseFloat(a) - parseFloat(b));

            sortedSentiments.forEach(sentiment => {
              totalsLabels.push(sentiment);
              totalsData.push(sentimentTotals[sentiment]);
              totalsColors.push(colorMap[sentiment] || 'rgb(108, 117, 125)'); // default gray
            });

            const totalsConfig = {
              type: 'bar',
              data: {
                labels: totalsLabels,
                datasets: [{
                  label: Drupal.t('Total Count'),
                  data: totalsData,
                  backgroundColor: totalsColors,
                  borderColor: totalsColors,
                  borderWidth: 1
                }]
              },
              options: {
                responsive: true,
                scales: {
                  y: {
                    beginAtZero: true
                  },
                  x: {
                    title: {
                      display: true,
                      text: Drupal.t('Sentiment Value')
                    }
                  }
                },
                plugins: {
                  legend: {
                    display: false // Hide legend since we only have one dataset
                  },
                  title: {
                    display: true,
                    text: Drupal.t('Total Sentiment Distribution')
                  }
                }
              }
            };

            new Chart(totalsCanvas.getContext('2d'), totalsConfig);
          }
        }

        // Create tags polar chart
        if (tagsCanvas && tagsCanvas.getContext && drupalSettings.ai_webform_sentiment.tag_counts) {
          tagsCanvas.once = tagsCanvas.once || {};

          if (!tagsCanvas.once.tagsPolarChart) {
            tagsCanvas.once.tagsPolarChart = true;

            const tagCounts = drupalSettings.ai_webform_sentiment.tag_counts;
            const tagsLabels = Object.keys(tagCounts);
            const tagsData = Object.values(tagCounts);

            // Generate colors for tags.
            const tagsColors = [];
            const baseColors = [
              'rgb(255, 99, 132)',
              'rgb(54, 162, 235)',
              'rgb(255, 205, 86)',
              'rgb(75, 192, 192)',
              'rgb(153, 102, 255)',
              'rgb(255, 159, 64)',
              'rgb(199, 199, 199)',
              'rgb(83, 102, 255)',
              'rgb(40, 167, 69)',
              'rgb(220, 53, 69)'
            ];

            for (let i = 0; i < tagsLabels.length; i++) {
              tagsColors.push(baseColors[i % baseColors.length]);
            }

            const tagsConfig = {
              type: 'polarArea',
              data: {
                labels: tagsLabels,
                datasets: [{
                  label: Drupal.t('Tag Usage'),
                  data: tagsData,
                  backgroundColor: tagsColors,
                  borderColor: tagsColors,
                  borderWidth: 1
                }]
              },
              options: {
                responsive: true,
                plugins: {
                  legend: {
                    position: 'right',
                  },
                  title: {
                    display: true,
                    text: Drupal.t('Tag Distribution')
                  }
                },
                scales: {
                  r: {
                    beginAtZero: true
                  }
                }
              }
            };

            new Chart(tagsCanvas.getContext('2d'), tagsConfig);
          }
        }
      }
    }
  };

})(jQuery, Drupal, drupalSettings);
