<?php

namespace Drupal\ai_webform_sentiment\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\webform\Entity\Webform;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for the sentiment analysis dashboard.
 */
class SentimentDashboardController extends ControllerBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new SentimentDashboardController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(Connection $database, EntityTypeManagerInterface $entity_type_manager) {
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Displays the sentiment analysis dashboard.
   *
   * @param \Drupal\webform\Entity\Webform $webform
   *   The webform entity.
   *
   * @return array
   *   The render array.
   */
  public function dashboard(Webform $webform) {
    // Calculate the date range (one month ago from today).
    $current_day = new DrupalDateTime();
    $current_day->setTime(23, 59, 59);
    $start_day = new DrupalDateTime();
    $start_day->modify('-1 month');
    $start_day->setTime(0, 0, 0);

    // Fetch unique sentiment values for this webform.
    $query = $this->database->select('ai_webform_sentiment', 'aws')
      ->fields('aws', ['sentiment'])
      ->condition('aws.webform_id', $webform->id())
      ->condition('aws.datetime', $start_day->format('Y-m-d H:i:s'), '>=')
      ->distinct();
    $unique_values = $query->execute()->fetchCol();

    // Calculate average sentiment for the date range.
    $avg_query = $this->database->select('ai_webform_sentiment', 'aws')
      ->condition('aws.webform_id', $webform->id())
      ->condition('aws.datetime', $start_day->format('Y-m-d H:i:s'), '>=');
    $avg_query->addExpression('AVG(sentiment)', 'avg_sentiment');
    $average_sentiment = $avg_query->execute()->fetchField();
    $average_sentiment = $average_sentiment ? round($average_sentiment, 2) : 0;
    $average_sentiment = number_format($average_sentiment, 2, '.', '') . ' / 5';

    // Calculate today's average sentiment.
    $today = new DrupalDateTime();
    $today_start = clone $today;
    $today_start->setTime(0, 0, 0);
    $today_end = clone $today;
    $today_end->setTime(23, 59, 59);
    $today_avg_query = $this->database->select('ai_webform_sentiment', 'aws')
      ->condition('aws.webform_id', $webform->id())
      ->condition('aws.datetime', $today_start->format('Y-m-d H:i:s'), '>=')
      ->condition('aws.datetime', $today_end->format('Y-m-d H:i:s'), '<=');
    $today_avg_query->addExpression('AVG(sentiment)', 'avg_sentiment');
    $daily_average_sentiment = $today_avg_query->execute()->fetchField();
    $daily_average_sentiment = $daily_average_sentiment ? round($daily_average_sentiment, 2) : 0;
    $daily_average_sentiment = number_format($daily_average_sentiment, 2, '.', '') . ' / 5';

    // Get daily averages for the chart.
    $daily_avg_query = $this->database->select('ai_webform_sentiment', 'aws')
      ->condition('aws.webform_id', $webform->id())
      ->condition('aws.datetime', $start_day->format('Y-m-d H:i:s'), '>=')
      ->condition('aws.datetime', $current_day->format('Y-m-d H:i:s'), '<=');
    $daily_avg_query->addExpression('DATE(datetime)', 'date');
    $daily_avg_query->addExpression('AVG(sentiment)', 'avg_sentiment');
    $daily_avg_query->groupBy('date');
    $daily_averages = $daily_avg_query->execute()->fetchAllKeyed();

    // Get total counts for each sentiment value for the date range.
    $totals_query = $this->database->select('ai_webform_sentiment', 'aws')
      ->condition('aws.webform_id', $webform->id())
      ->condition('aws.datetime', $start_day->format('Y-m-d H:i:s'), '>=');
    $totals_query->addExpression('COUNT(sentiment)', 'count');
    $totals_query->addField('aws', 'sentiment');
    $totals_query->groupBy('sentiment');
    $totals_query->orderBy('sentiment', 'ASC');
    $sentiment_totals = $totals_query->execute()->fetchAllKeyed();

    // Get tag counts for the date range.
    $tags_query = $this->database->select('ai_webform_sentiment_tags', 'awst');
    $tags_query->join('ai_webform_sentiment', 'aws', 'awst.sentiment_id = aws.sentiment_id');
    $tags_query->condition('awst.webform_id', $webform->id());
    $tags_query->condition('aws.datetime', $start_day->format('Y-m-d H:i:s'), '>=');
    $tags_query->addExpression('COUNT(awst.tag)', 'count');
    $tags_query->addField('awst', 'tag');
    $tags_query->groupBy('awst.tag');
    $tags_query->orderBy('count', 'DESC');
    // Limit to top 20 tags.
    $tags_query->range(0, 20);
    $tag_counts = $tags_query->execute()->fetchAllKeyed();

    // Prepare data arrays.
    $days_list = [];
    $sentiment_data = [];
    $daily_avg_data = [];

    // Initialize sentiment data arrays.
    foreach ($unique_values as $sentiment) {
      $sentiment_data[$sentiment] = [];
    }

    // Loop through each day and collect sentiment data.
    $iteration_day = clone $start_day;
    while ($iteration_day <= $current_day) {
      $formatted_day = $iteration_day->format('Y-m-d');
      $days_list[] = $iteration_day->format('M d');

      $next_day = clone $iteration_day;
      $next_day->modify('+1 day');

      // For each sentiment value, get count for this day.
      foreach ($unique_values as $sentiment) {
        $query = $this->database->select('ai_webform_sentiment', 'aws')
          ->condition('aws.webform_id', $webform->id())
          ->condition('aws.sentiment', $sentiment)
          ->condition('aws.datetime', $iteration_day->format('Y-m-d 00:00:00'), '>=')
          ->condition('aws.datetime', $next_day->format('Y-m-d 00:00:00'), '<');
        $query->addExpression('COUNT(*)', 'count');
        $count = $query->execute()->fetchField() ?: 0;

        $sentiment_data[$sentiment][] = $count;
      }

      // Add daily average for this day.
      $daily_avg = isset($daily_averages[$formatted_day]) ? round($daily_averages[$formatted_day], 2) : NULL;
      $daily_avg_data[] = $daily_avg;

      // Move to the next day.
      $iteration_day->modify('+1 day');
    }

    // Add chart data to drupalSettings.
    $chart_settings = [
      'days' => $days_list,
      'webform_id' => $webform->id(),
      'daily_averages' => $daily_avg_data,
      'sentiment_totals' => $sentiment_totals,
      'tag_counts' => $tag_counts,
    ];

    // Add sentiment data.
    foreach ($unique_values as $index => $sentiment) {
      $chart_settings['data_' . $index] = $sentiment_data[$sentiment];
      $chart_settings['label_' . $index] = $sentiment;
    }

    // Add the number of sentiments for JavaScript to loop through.
    $chart_settings['sentiment_count'] = count($unique_values);

    // Attach JS and settings.
    $build = [
      '#theme' => 'ai_webform_sentiment_dashboard',
      '#webform_id' => $webform->id(),
      '#average_sentiment' => $average_sentiment,
      '#daily_average_sentiment' => $daily_average_sentiment,
      '#start_date' => $start_day->format('d M Y'),
      '#current_date' => $current_day->format('d M Y'),
      '#attached' => [
        'library' => [
          'ai_webform_sentiment/dashboard',
        ],
        'drupalSettings' => [
          'ai_webform_sentiment' => $chart_settings,
        ],
      ],
    ];

    return $build;
  }

}
