<?php

namespace Drupal\ai_webform_sentiment\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Webform submission AI tagging handler.
 *
 * @WebformHandler(
 *   id = "ai_submission_tagger",
 *   label = @Translation("AI Submission Tagger"),
 *   category = @Translation("AI"),
 *   description = @Translation("Analyzes webform submissions for tags using AI."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class SubmissionTaggerWebformHandler extends WebformHandlerBase {

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The AI provider manager.
   *
   * @var \Drupal\ai\AiProviderPluginManager
   */
  protected $aiProviderManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->queueFactory = $container->get('queue');
    $instance->aiProviderManager = $container->get('ai.provider');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'prompt' => '
INSTRUCTION:
--------------
You are a submission tagger.
You will analyze the content and provide relevant tags for categorization.
Return only comma-separated tags, no explanation. Maximum 5 tags.

CONTENT:
---------
[submission]
      ',
      'llm_model' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $llm_model_options = $this->aiProviderManager->getSimpleProviderModelOptions('chat');
    array_shift($llm_model_options);
    $form['llm_model'] = [
      '#type' => 'select',
      "#empty_option" => $this->t('-- Default from AI module (chat) --'),
      '#title' => $this->t('LLM Model'),
      '#default_value' => $this->configuration['llm_model'],
      '#options' => $llm_model_options,
      '#description' => $this->t('Select which provider to use for this plugin. See the <a href=":link">Provider overview</a> for details about each provider.', [':link' => '/admin/config/ai/providers']),
    ];

    $form['prompt'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Prompt'),
      '#description' => $this->t("Add your prompt here, don't forget to use the [submission] for the submission details."),
      '#default_value' => $this->configuration['prompt'],
      '#required' => TRUE,
      '#rows' => 5,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['llm_model'] = $form_state->getValue('llm_model');
    $this->configuration['prompt'] = $form_state->getValue('prompt');
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    // Don't process draft submissions.
    if ($webform_submission->isDraft()) {
      return;
    }

    // Add to queue for processing.
    $queue = $this->queueFactory->get('ai_tag_submissions');
    $item = [
      'webform_id' => $webform_submission->getWebform()->id(),
      'submission_id' => $webform_submission->id(),
      'handler_id' => $this->getHandlerId(),
    ];

    $queue->createItem($item);
  }

}
