<?php

namespace Drupal\ai_webform_sentiment;

use Drupal\ai\AiProviderPluginManager;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\Component\Datetime\TimeInterface;

/**
 * Processes the submission tagging queue.
 */
class SubmissionTaggerProcessor {
  use StringTranslationTrait;

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The AI provider manager.
   *
   * @var \Drupal\ai\AiProviderPluginManager
   */
  protected $aiProviderManager;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a new SubmissionTaggerProcessor object.
   *
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\ai\AiProviderPluginManager $ai_provider_manager
   *   The AI provider manager.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    QueueFactory $queue_factory,
    EntityTypeManagerInterface $entity_type_manager,
    AiProviderPluginManager $ai_provider_manager,
    Connection $database,
    TimeInterface $time,
    LoggerChannelFactoryInterface $logger_factory,
  ) {
    $this->queueFactory = $queue_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->aiProviderManager = $ai_provider_manager;
    $this->database = $database;
    $this->time = $time;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * Process submission tagging queue items.
   */
  public function processQueue() {
    $queue = $this->queueFactory->get('ai_tag_submissions');

    // Debug: Check queue size.
    $queue_size = $queue->numberOfItems();
    $this->loggerFactory->get('ai_webform_sentiment')->notice('Tagging queue size: @size', ['@size' => $queue_size]);

    // Process items from the queue.
    $item_count = 0;
    while ($item = $queue->claimItem()) {
      try {
        $this->processItem($item->data);
        $queue->deleteItem($item);
        $item_count++;
      }
      catch (\Exception $e) {
        // Log the exception and release the item back to the queue.
        $this->loggerFactory->get('ai_webform_sentiment')->error('Error processing submission tagging: @message', [
          '@message' => $e->getMessage(),
        ]);
        $queue->releaseItem($item);
      }
    }

    $this->loggerFactory->get('ai_webform_sentiment')->notice('Processed @count tagging items', ['@count' => $item_count]);
    return $item_count;
  }

  /**
   * Process a single queue item.
   *
   * @param array $data
   *   The queue item data.
   */
  public function processItem(array $data) {
    // 1. Load the webform submission.
    $submission = $this->entityTypeManager->getStorage('webform_submission')
      ->load($data['submission_id']);

    if (!$submission) {
      $this->loggerFactory->get('ai_webform_sentiment')->error('Could not load webform submission: @id', [
        '@id' => $data['submission_id'],
      ]);
      return;
    }

    // 2. Load the handler configuration to get the prompt.
    $webform = $submission->getWebform();
    $handler = $webform->getHandler($data['handler_id']);
    $config = $handler->getConfiguration();
    $prompt = $handler->getConfiguration()['settings']['prompt'];
    $ai_provider_model = $handler->getConfiguration()['settings']['llm_model'] ?? '';

    // 3. Replace simple [submission] placeholder if it exists.
    if (strpos($prompt, '[submission]') !== FALSE) {
      $submission_data = '';
      $submission_form_data = $submission->getData();
      foreach ($submission_form_data as $key => $value) {
        if (is_array($value)) {
          $value = implode(', ', $value);
        }
        $submission_data .= $key . ': ' . $value . "\n";
      }
      $prompt = str_replace('[submission]', $submission_data, $prompt);
    }

    // 4. Load an AI provider.
    if ($ai_provider_model === '') {
      $default_provider = $this->aiProviderManager->getDefaultProviderForOperationType('chat');
      $ai_provider_model = $default_provider['provider_id'] . '__' . $default_provider['model_id'];
      $ai_model_to_use = $default_provider['model_id'];
    }
    else {
      $parts = explode('__', $ai_provider_model);
      $ai_model_to_use = $parts[1];
    }

    // 5. Call the AI method.
    try {
      $provider = $this->aiProviderManager->loadProviderFromSimpleOption($ai_provider_model);
      $temp = $config['llm_temp'] ?? 0.5;
      $provider->setConfiguration(['temperature' => (float) $temp]);
      $input = new ChatInput([
        new ChatMessage('user', $prompt),
      ]);
      $output = $provider->chat($input, $ai_model_to_use, ['ai_search_block']);
      $response = $output->getNormalized()->getText();

      // Parse tags from response (comma-separated).
      $tags = explode(',', $response);
      $tags = array_map('trim', $tags);
      // Remove empty tags.
      $tags = array_filter($tags);

      // 6. First get or create the sentiment record.
      $sentiment_id = $this->getOrCreateSentimentRecord($data);

      // 7. Save tags in the database.
      foreach ($tags as $tag) {
        $this->database->insert('ai_webform_sentiment_tags')
          ->fields([
            'sentiment_id' => $sentiment_id,
            'webform_id' => $data['webform_id'],
            'tag' => $tag,
          ])
          ->execute();
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('ai_webform_sentiment')->error('Error during AI tagging: @message', [
        '@message' => $e->getMessage(),
      ]);
      throw $e;
    }
  }

  /**
   * Get or create a sentiment record for the submission.
   *
   * @param array $data
   *   The queue item data.
   *
   * @return int
   *   The sentiment_id.
   */
  protected function getOrCreateSentimentRecord(array $data) {
    // Check if sentiment record already exists.
    $sentiment_id = $this->database->select('ai_webform_sentiment', 'aws')
      ->fields('aws', ['sentiment_id'])
      ->condition('webform_id', $data['webform_id'])
      ->condition('submission_id', $data['submission_id'])
      ->execute()
      ->fetchField();

    if ($sentiment_id) {
      return $sentiment_id;
    }

    // Create a new sentiment record if it doesn't exist.
    $current_time = new DrupalDateTime();
    $sentiment_id = $this->database->insert('ai_webform_sentiment')
      ->fields([
        'webform_id' => $data['webform_id'],
        'submission_id' => $data['submission_id'],
        'datetime' => $current_time->format(DateTimeItemInterface::DATETIME_STORAGE_FORMAT),
        // Default sentiment value.
        'sentiment' => '0',
      ])
      ->execute();

    return $sentiment_id;
  }

}
